
#include "LivoxMid100Laser.h"
#include <glog/logging.h>
RegisterLaser(LivoxMid100)


CLivoxMid100Laser::CLivoxMid100Laser(int id, Laser_proto::laser_parameter laser_param)
	:CLivoxLaser(id, laser_param)
{
	m_frame_maxnum = laser_param.frame_num();
	if (laser_param.type() == "LivoxMid100")
	{
		std::string sn = laser_param.sn();
		std::string sn1 = sn, sn2 = sn, sn3 = sn;
		sn1 += "1";
		sn2 += "2";
		sn3 += "3";
		g_sn_laser.insert(std::make_pair(sn1, this));
		g_sn_laser.insert(std::make_pair(sn2, this));
		g_sn_laser.insert(std::make_pair(sn3, this));
	}
}

void CLivoxMid100Laser::UpdataHandle()
{
	std::string sn = m_laser_param.sn();
	std::string sn1 = sn, sn2 = sn, sn3 = sn;
	sn1 += "1";
	sn2 += "2";
	sn3 += "3";
	if (g_sn_handle.find(sn1) != g_sn_handle.end())
	{
		m_handle1 = g_sn_handle[sn1];
	}
	if (g_sn_handle.find(sn2) != g_sn_handle.end())
	{
		m_handle2 = g_sn_handle[sn2];
	}
	if (g_sn_handle.find(sn3) != g_sn_handle.end())
	{
		m_handle3 = g_sn_handle[sn3];
	}
}
bool CLivoxMid100Laser::IsScanComplete()
{
    if(mp_laser_task==NULL)
    {
        return false;
    }
    else {
        int frame_count=mp_laser_task->get_task_points_number();
        return g_count[m_handle1] >= frame_count && g_count[m_handle2] >= frame_count
            && g_count[m_handle2] >= frame_count;
    }
}

eLaserStatu CLivoxMid100Laser::GetStatu()
{
    bool cond1=g_devices[m_handle1].device_state == kDeviceStateConnect ||
               g_devices[m_handle1].device_state == kDeviceStateSampling;
    bool cond2=g_devices[m_handle2].device_state == kDeviceStateConnect ||
               g_devices[m_handle2].device_state == kDeviceStateSampling;
    bool cond3=g_devices[m_handle3].device_state == kDeviceStateConnect ||
               g_devices[m_handle3].device_state == kDeviceStateSampling;
    if (cond1 && cond2 && cond3) {
        if(m_queue_laser_data.size()==0 && m_bscan_start==false)
            m_statu = eLaser_ready;
        else
            m_statu = eLaser_busy;
    }
    else {
        m_statu = eLaser_disconnect;
    }


    return m_statu;

}

bool CLivoxMid100Laser::Connect()
{
    //设置点云变换矩阵
    double matrix[12]={0};
    matrix[0]=m_laser_param.mat_r00();
    matrix[1]=m_laser_param.mat_r01();
    matrix[2]=m_laser_param.mat_r02();
    matrix[3]=m_laser_param.mat_r03();

    matrix[4]=m_laser_param.mat_r10();
    matrix[5]=m_laser_param.mat_r11();
    matrix[6]=m_laser_param.mat_r12();
    matrix[7]=m_laser_param.mat_r13();

    matrix[8]=m_laser_param.mat_r20();
    matrix[9]=m_laser_param.mat_r21();
    matrix[10]=m_laser_param.mat_r22();
    matrix[11]=m_laser_param.mat_r23();
    SetMetrix(matrix);
    return CLivoxLaser::Connect();
}
void CLivoxMid100Laser::Disconnect()
{
    CLivoxLaser::Disconnect();
}

bool CLivoxMid100Laser::Stop()
{
    LOG(INFO)<<" livoxmid100 stoped !!!"<<m_laser_param.sn();
    return CLivoxLaser::Stop();
}
bool CLivoxMid100Laser::Start()
{
		LOG(INFO) << " livoxMid100 start :" << m_laser_param.sn();
		m_queue_livox_data.clear();
		g_count[m_handle1] = 0;
		g_count[m_handle2] = 0;
		g_count[m_handle3] = 0;
		return Laser_base::Start();
}

Error_manager CLivoxMid100Laser::execute_task(Task_Base* p_laser_task)
{
    Error_manager t_error_manager;
//检查指针
    if (p_laser_task == NULL) {
        return Error_manager(Error_code::POINTER_IS_NULL, Error_level::MINOR_ERROR,
                             "Laser_base::porform_task failed, POINTER_IS_NULL");
    }
    if (p_laser_task->get_task_type() != LASER_TASK) {
        return Error_manager(Error_code::LASER_TASK_TYPE_ERROR, Error_level::MINOR_ERROR,
                             "laser task type error");
    }


//接受任务，并将任务的状态改为TASK_SIGNED已签收
    mp_laser_task = (Laser_task *) p_laser_task;
    mp_laser_task->set_task_statu(TASK_SIGNED);


//检查消息内容是否正确，
//检查点云指针
    if (mp_laser_task->get_task_point3d_cloud_vector().get() == NULL) {
        t_error_manager.error_manager_reset(Error_code::POINTER_IS_NULL,
                                            Error_level::MINOR_ERROR,
                                            "Laser_base::porform_task failed, POINTER_IS_NULL");
    }
    else{
        m_frame_maxnum=mp_laser_task->get_task_points_number();
//将任务的状态改为 TASK_WORKING 处理中
        mp_laser_task->set_task_statu(TASK_WORKING);
        std::string save_path = mp_laser_task->get_save_path();
        SetSaveDir(save_path);
//启动雷达扫描
        Start();
    }
//返回错误码
    if (t_error_manager != Error_code::SUCCESS) {
        mp_laser_task->set_task_error_manager(t_error_manager);
    }
    return t_error_manager;
}
CLivoxMid100Laser::~CLivoxMid100Laser()
{
}
