
//laser_task_command，是雷达任务指令的相关功能
//功能：用作应用层向laser模块传输任务，的指令消息

//用法：应用层直接调用laser的接口函数，并将Laser_task类作为参数传递
//CLaser类则按照Laser_task的功能码和指定的参数，执行相应的功能
//并将结果填充到Laser_task，返回给应用层

#include "laser_task_command.h"
//构造函数
Laser_task::Laser_task()
{
    m_task_type = LASER_TASK;
    m_task_statu = TASK_CREATED;
    //m_task_statu_information默认为空

    m_task_points_number = 0;
    pm_task_point3d_cloud_vector = NULL;
    //m_task_error_manager 默认为空
}
//析构函数
Laser_task::~Laser_task()
{

}

//初始化任务单，必须初始化之后才可以使用，（必选参数）
//input:task_type 任务类型
// input:task_statu任务状态
// output:p_task_point3d_cloud_vector三维点云容器指针
//注：task_points_number默认为0，如果为0，则使用laser默认的点云的采集数量
Error_manager Laser_task::task_init( Task_statu task_statu,
                                     pcl::PointCloud<pcl::PointXYZ>::Ptr p_task_point3d_cloud_vector,
                                     std::mutex* lock)
{
    if(p_task_point3d_cloud_vector.get() == NULL)
    {
        return Error_manager(Error_code::POINTER_IS_NULL, Error_level::MINOR_ERROR,
                             "Laser_task::task_init failed");
    }
    if(lock==NULL)
    {
        return Error_manager(PARAMETER_ERROR,NORMAL,"laser task input lock is null");
    }
    mp_cloud_lock=lock;
    m_task_statu = task_statu;
    m_task_statu_information.clear();

    m_task_points_number = 0;
    pm_task_point3d_cloud_vector = p_task_point3d_cloud_vector;
    m_task_error_manager.error_manager_clear_all();

    return Error_code::SUCCESS;
}

//初始化任务单，必须初始化之后才可以使用，（可选参数）
//    input:task_type 任务类型
//    input:task_statu任务状态
//    input:task_statu_information状态说明
//    input:task_points_number点云的采集数量
//    output:p_task_point3d_cloud_vector三维点云容器指针
//注：task_points_number默认为0，如果为0，则使用laser默认的点云的采集数量
Error_manager Laser_task::task_init(Task_statu task_statu,
                                    std::string & task_statu_information,
                                    int task_points_number,
                                    pcl::PointCloud<pcl::PointXYZ>::Ptr p_task_point3d_cloud_vector,
                                    std::mutex* lock)
{
    if(p_task_point3d_cloud_vector.get() == NULL)
    {
        return Error_manager(Error_code::POINTER_IS_NULL, Error_level::MINOR_ERROR,
                             "Laser_task::task_init failed");
    }
    if(lock==NULL)
    {
        return Error_manager(PARAMETER_ERROR,NORMAL,"laser task input lock is null");
    }
    mp_cloud_lock=lock;
    m_task_statu = task_statu;
    m_task_statu_information = task_statu_information;

    m_task_points_number = task_points_number;
    pm_task_point3d_cloud_vector = p_task_point3d_cloud_vector;
    m_task_error_manager.error_manager_clear_all();

    return Error_code::SUCCESS;
}

//push点云
Error_manager Laser_task::push_point(pcl::PointXYZ point)
{
    if(mp_cloud_lock==NULL)
    {
        return Error_manager(PARAMETER_ERROR,NORMAL,"push_point laser task input lock is null");
    }
    if(pm_task_point3d_cloud_vector.get()==NULL)
    {
        return Error_manager(Error_code::POINTER_IS_NULL, Error_level::MINOR_ERROR,
                            "Laser_task::push_point failed");
    }
    mp_cloud_lock->lock();
    pm_task_point3d_cloud_vector->push_back(point);
    mp_cloud_lock->unlock();
    return SUCCESS;
}

//获取 点云的采集数量
int Laser_task::get_task_points_number()
{
    return m_task_points_number;
}
//获取 三维点云容器
pcl::PointCloud<pcl::PointXYZ>::Ptr Laser_task::get_task_point3d_cloud_vector()
{
    return pm_task_point3d_cloud_vector;
}
//获取 错误码
Error_manager Laser_task::get_task_error_manager()
{
    return m_task_error_manager;
}

//设置 任务状态
void Laser_task::set_task_statu(Task_statu task_statu)
{
    m_task_statu = task_statu;
}
//设置 任务状态说明
void Laser_task::set_task_statu_information(std::string & task_statu_information)
{
    m_task_statu_information = task_statu_information;
}
//设置 点云的采集数量
void Laser_task::set_task_points_number(int task_points_number)
{
    m_task_points_number = task_points_number;
}
//设置 三维点云容器
void Laser_task::set_task_point3d_cloud_vector(pcl::PointCloud<pcl::PointXYZ>::Ptr p_task_point3d_cloud_vector)
{
    pm_task_point3d_cloud_vector = p_task_point3d_cloud_vector;
}
//设置 错误码
void Laser_task::set_task_error_manager(Error_manager & error_manager)
{
    m_task_error_manager = error_manager;
}

//获取采集的点云保存路径
std::string Laser_task::get_save_path()
{
    return m_save_path;
}
//设置采集的点云保存路径
void Laser_task::set_save_path(std::string save_path)
{
    m_save_path=save_path;
}






